# This is a simple tool to procedurally generate a three-color gradient on a series of faces.
# Gradientator by steve r is licensed under a Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.

module SR_Gradientator

	# Setting VERBOSE to nil is a simple speed boost tip from Dan Rathbun on the Sketchucation forums.
	
	$VERBOSE = nil
	
	# Version check code adopted from Google.
	
	version_required = 7.0

	if (Sketchup.version.to_f < version_required) 
		UI.messagebox("You must have Sketchup " + version_required.to_s + " to run this plugin. Visit sketchup.google.com to upgrade.")
		return
	end

	def self.gradientate()
		
		model = Sketchup.active_model
		
		# Combined with commit_operation later on, makes it so 'Undo' undoes the entire operation.
		
		model.start_operation "Gradientation",true
		
		entities = model.active_entities
		selection = model.selection
		
		# Cycle through selection, add things that happen to be faces to a collection.
		
		faces = selection.select{ |e| e.is_a?(Sketchup::Face) }
		
		numberfaces = faces.length
		
		# Returns an error if the user didn't select any faces, or if they only selected non-faces (which have since been scrubbed from the selection).
		
		if numberfaces == 0
		
			UI.messagebox("No faces selected!")
			return nil
		
		end
		
		# Prompt for user to select colors in gradient and gradient method.
		
		colors = Sketchup::Color.names
		colorsarray = colors.collect{ |c| c+"|" }
		# displaynames = materials.collect {|m| m.display_name}
		prompts = ["Choose gradient color 1:","Choose gradient color 2:","Choose gradient color 3:","Choose gradient order method:"]
		values = ["Red","Yellow","Lime","Distance from Origin"]
		enums = [colorsarray,colorsarray,colorsarray,"Random|Distance from Origin"]
		results = inputbox prompts, values, enums, $exStrings.GetString("Gradientator!")
		return if not results
		
		# Determines the three colors (as an array of RGB values) that the user wants to gradient between.
		
		colorone = Sketchup::Color.new results[0]
		colortwo = Sketchup::Color.new results[1]
		colorthree = Sketchup::Color.new results[2]
		
		# Determines how much to alter the RGB values of the colors when going from colorone to colortwo (variables ending with a) and from colortwo to colorthree (variables ending with b).
		
		redshifta = (colortwo.red - colorone.red)/(numberfaces/2)
		redshiftb = (colorthree.red - colortwo.red)/(numberfaces/2)
		greenshifta = (colortwo.green - colorone.green)/(numberfaces/2)
		greenshiftb = (colorthree.green - colortwo.green)/(numberfaces/2)
		blueshifta = (colortwo.blue - colorone.blue)/(numberfaces/2)
		blueshiftb = (colorthree.blue - colortwo.blue)/(numberfaces/2)
		
		redbalance = colorone.red.to_f
		greenbalance = colorone.green.to_f
		bluebalance = colorone.blue.to_f
		
		# The counter will later let us know when we're halfway done with the gradient, and should start using the 'b' values.
		
		counter = 0
		
		# Sorts the previously unordered faces object into an object ordered by distance from origin, if desired by the user. Code concept provided by Thom Thom and sdmitch from the Sketchucation forums.
		
		if results[3] == "Distance from Origin"
		
			stack = {}
			faces.each{|face| stack[face]=ORIGIN.distance(face.bounds.center)}
			faces = stack.sort { |a,b| a[1] <=> b[1] }.map { |n| n[0] }
			
		end
		
		# Cycles through everything in the faces collection, applying the correct material to each.
		
		faces.each{ |f|
			
			f.material = [redbalance.round, greenbalance.round, bluebalance.round]
			
			if counter < (numberfaces/2)
			
				redbalance += redshifta
				greenbalance += greenshifta
				bluebalance += blueshifta
			
			else
			
				redbalance += redshiftb
				greenbalance += greenshiftb
				bluebalance += blueshiftb
				
			end
			
			counter += 1
			
		}
		
		model.commit_operation
		
	end

end # End of Module SR_Gradientator

# Add Gradientate option to plugins menu.

if not file_loaded?(File.basename(__FILE__))
	UI.menu("Plugins").add_item("Gradientate") { SR_Gradientator::gradientate() }
end

file_loaded(File.basename(__FILE__))   # Mark the script loaded.